package movieorganizer.imdbrequester;

import java.util.ArrayList;

import movieorganizer.userinterface.ImdbEntry;
import movieorganizer.userinterface.Movie;

/** This class is able to request IMDB-information from a given server. */
public abstract class AImdbRequester {
    /** Returns all IMDB-Entries for a given movie.
     * @param movie the movie.
     * @return the IMDB-entries for the given movie. */
    public abstract ArrayList<ImdbEntry> getEntries(Movie movie);

    /** Return all IMDB-Entries for the given query.
     * @param query the query.
     * @return the list of IMDB-entries for the given query. */
    protected abstract ArrayList<ImdbEntry> sendRequest(String query);

    /** The abstract IMDB-requester. */
    public AImdbRequester() {
    }

    /**
     * Returns the cleaned movie-title, this means a String which is ready for
     * an IMDB-request: only words and numbers separated by one blank and all in
     * lowercase.
     * @param movieTitle the given movieTitle.
     * @return the cleaned movie-title.
     */
    public static final String cleanTitle(final String movieTitle) {
        String result = movieTitle.toLowerCase();
        // Remove all characters except a...z,0..9 and blank.
        result = result.replaceAll(
                "[^\\p{Alpha}\\p{Digit}\\p{Blank}]|[-]", " ");
        // Remove all blank...blank.
        String blanks = "";
        for (int i = 0; i < result.length(); i++) {
            blanks += " ";
        }
        for (int i = 0; i < result.length(); i++) {
            result = result.replace(blanks, " ");
            blanks = blanks.substring(0, Math.min(Math.max(blanks.length() - 1,
                    2), blanks.length()));
        }
        return result;
    }

    /**
     * Send requests starting with the _last_ n words to n-1, ..., n-k until
     *  the server gets results. Return the IMDB-entries and stop further
     *  requests.
     * @param input the input-query.
     * @param k the maximum number of words at the beginning which should be
     *          not be part of the query.
     * @return the IMDB-results.
     */
    protected final ArrayList<ImdbEntry> sendBlockRequests(final String input,
            final int k) {
        ArrayList<ImdbEntry> result = null;
        BinSearchResult request = null;
        int bestQueryLength = 0;
        String[] words = input.split(" ");
        int n = words.length;
        for (int i = 0; i <= k; i++) {
            request = null;
            // Prepare the query.
            String query = "";
            for (int j = i; j < n; j++) {
                query += words[j] + " ";
            }
            if (query.endsWith(" ")) { // Remove the last blank.
                query = query.substring(0, query.length() - 1);
            }
            if (query.length() > 0) {
                //System.out.println("Starting binary search with: " + query);
                request = binSearchRequest(query);
            }

            // Take the longest successful query which is most to the right.
            if (request != null && request.imdbEntries != null
                    && request.imdbEntries.size() > 0
                    && request.numberOfWords >= bestQueryLength) {
                result = request.imdbEntries;
                bestQueryLength = request.numberOfWords;
            }
        }
        return result;
    }

    /** This class represents a result from the binary-search. */
    private final class BinSearchResult {
        /** The IMDB-entries. */
        private ArrayList<ImdbEntry> imdbEntries;
        /** The number of words which lead to these results. */
        private int numberOfWords;

        /** @param entries the IMDB-entries.
         * @param numberWords the number of words which lead to these result. */
        private BinSearchResult(final ArrayList<ImdbEntry> entries,
                final int numberWords) {
            this.imdbEntries = entries;
            this.numberOfWords = numberWords;
        }
    }

    /**
     * Search the perfect query (neither too short nor too long) with a
     * binary search.
     * @param input the input-query.
     * @return the list of movies found with the perfect query.
     * */
    protected final BinSearchResult binSearchRequest(final String input) {
        BinSearchResult result = null;
        String[] words = input.split(" ");
        int l = 0;
        int r = words.length - 1;
        int m = 0;
        while (l <= r) {
            //System.out.println("l: " + l + "; r: " + r);
            m = (l + r) / 2;
            //System.out.println("m: " + m);
            String query = "";
            for (int i = 0; i <= m; i++) {
                query += words[i] + " ";
            }
            if (query.endsWith(" ")) { // Remove the last blank.
                query = query.substring(0, query.length() - 1);
            }

            ArrayList<ImdbEntry> request = sendRequest(query);
            // Check if the query was better than the previous queries.
            if (request != null && request.size() > 0) {
                l = m + 1;
                int numberOfWords = query.split(" ").length;
                result = new BinSearchResult(request, numberOfWords);
            } else {
                r = m - 1; // We found nothing better.
            }
        }
        return result;
    }

}
