package movieorganizer.userinterface;

import java.io.File;

import java.net.URI;
import java.util.ArrayList;

import movieorganizer.imdbrequester.AImdbRequester;
import movieorganizer.imdbrequester.StrombImdbRequester;

/**
 * Everything about a single movie: file on disk to which it belongs, metadata
 * from IMDB-entry.
 */
public class Movie {
    /** Name of movie file. */
    private String fileName;
    /** Path to that file. */
    private String pathToFile;
    /** The selected title of the movie. */
    private String title;
    /** The selected director of the movie. */
    private String director;
    /** The selected year of the movie. */
    private int year;
    /** The selected rating of the movie. */
    private float rating;
    /** The selected genres of the movie. */
    private ArrayList<String> genres;
    /** The selected duration of the movie. */
    private int duration;
    /** All IMDB-entries of the movie. */
    private ArrayList<ImdbEntry> imdbEntries;

    /** The standard constructor is needed for the XmlEncoder and XmlDecoder. */
    public Movie() {
    }

    /** Create a movie-object with its filename and its path.
     * @param fName the filename of the movie.
     * @param path the path to the movie-file. */
    public Movie(final String fName, final String path) {
        this.fileName = fName;
        this.pathToFile = path;
        title = AImdbRequester.cleanTitle(getTitleFromFileName());
        imdbEntries = new ArrayList<ImdbEntry>();
    }

    /** @return the filename. */
    public final String getFileName() {
        return this.fileName;
    }

    /** @param f the new filename. */
    public final void setFileName(final String f) {
        this.fileName = f;
    }

    /** @return the path. */
    public final String getPath() {
        return this.pathToFile;
    }

    /** @param p the new path. */
    public final void setPath(final String p) {
        this.pathToFile = p;
    }

    /** @return the title. */
    public final String getTitle() {
        return this.title;
    }

    /** @param t the new title. */
    public final void setTitle(final String t) {
        this.title = t;
    }

    /** @return the director. */
    public final String getDirector() {
        return this.director;
    }

    /** @param d the new director. */
    public final void setDirector(final String d) {
        this.director = d;
    }

    /** @return the year. */
    public final int getYear() {
        return this.year;
    }

    /** @param y the new year. */
    public final void setYear(final int y) {
        this.year = y;
    }

    /** @return the rating. */
    public final Float getRating() {
        return this.rating;
    }

    /** @param r the new rating. */
    public final void setRating(final Float r) {
        this.rating = r;
    }

    /** @return the genres. */
    public final ArrayList<String> getGenres() {
        return this.genres;
    }

    /** @param g the new genres. */
    public final void setGenres(final ArrayList<String> g) {
        this.genres = g;
    }

    /** @return the duration. */
    public final int getDuration() {
        return this.duration;
    }

    /** @param d the new duration. */
    public final void setDuration(final int d) {
        this.duration = d;
    }

    /** @return the IMDB-entries. */
    public final ArrayList<ImdbEntry> getImdbEntries() {
        return this.imdbEntries;
    }

    /** @param entries the new IMDB-entries. */
    public final void setImdbEntries(final ArrayList<ImdbEntry> entries) {
        this.imdbEntries = entries;
    }

    /** Returns the URI of the path (needed to open the movies folder).
     * @return the URI of the path. */
    public final URI getURI() {
        String result = pathToFile;
        if (result.endsWith(fileName)) {
            result = result.substring(0, result.length() - 1
                    - fileName.length());
        }
        File folder = new File(result);
        URI adr = folder.toURI();
        return adr;
    }

    /**
     * Gets the metadata for this movie. */
    public final void getMetaDataFromIMDB() {
        //System.out.println("Getting metadata from " + getTitle());
        AImdbRequester imdbRequester = new StrombImdbRequester();
        imdbEntries = imdbRequester.getEntries(this);

        // If we have at least one entry, we select the first one.
        if ((imdbEntries != null) && (imdbEntries.size() > 0)) {
            ImdbEntry e = imdbEntries.get(0);
            title = e.getTitle();
            director = e.getDirector();
            year = e.getYear();
            genres = e.getGenres();
            rating = e.getRating();
            duration = e.getDuration();
        } else {
            imdbEntries = new ArrayList<ImdbEntry>();
            fillMovieWithEmptyInformation(getTitle());
        }
    }

    /**
     * Fill movie information with empty entries. This is for the case that the
     * IMDB-request did not work.
     * @param t the title.
     */
    public final void fillMovieWithEmptyInformation(final String t) {
        this.setDirector(Settings.DEF_DIRECTOR);
        this.setDuration(Settings.DEF_DURATION);
        this.setGenres(new ArrayList<String>());
        this.setRating(Settings.DEF_RATING);
        this.setTitle(t);
        this.setYear(Settings.DEF_YEAR);
    }

    /** @return the title in lower-case without file-extension. */
    private String getTitleFromFileName() {
        return (this.fileName.substring(0, this.fileName.lastIndexOf(".")))
                .toLowerCase();
    }

    /** @return the information given for this movie. */
    public final String toString() {
        return ("{" + this.pathToFile + ", " + this.fileName + ", "
                + this.title + ", " + this.director + ", " + this.year + ", "
                + this.rating + ", " + this.genres + ", " + this.duration
                + "}");
    }

}
