package movieorganizer.userinterface;
import java.util.HashMap;

import javax.swing.table.AbstractTableModel;

/**
 * We define our own TableModel, the displayed information are: Title, Rating,
 * Year, Duration, Director, Genres.
 */
public class MovieTableModel extends AbstractTableModel {

    /** The default serial-version-UID. */
    private static final long serialVersionUID = 1L;
    /** The table header. */
    private String[] tableHeader = {"Title", "Rating", "Year", "Duration",
            "Director", "Genres" };
    /** The title column. */
    public static final int COL_TITLE = 0;
    /** The rating column. */
    public static final int COL_RATING = 1;
    /** The year column. */
    public static final int COL_YEAR = 2;
    /** The duration column. */
    public static final int COL_DURATION = 3;
    /** The director column. */
    public static final int COL_DIRECTOR = 4;
    /** The genres column. */
    public static final int COL_GENRES = 5;
    /** The corresponding movie object. */
    public static final int COL_MOVIE = 6;
    /** Here is our representation for the movies.
     * TODO(mirko): change this to a 2dim-vector. */
    private Object[][] movies = new Object[0][0];

    @Override
    public final String getColumnName(final int c) {
        return tableHeader[c];
    }

    @Override
    public final int getColumnCount() {
        return tableHeader.length;
    }

    @Override
    public final int getRowCount() {
        return movies.length;
    }

    @Override
    public final Object getValueAt(final int arg0, final int arg1) {
        return movies[arg0][arg1];
    }

    /** We want to be able to get the complete movie-object from a selected row.
     * @param row the row.
     * @return the movie. */
    public final Movie getMovieFromRow(final int row) {
        return (Movie) movies[row][COL_MOVIE];
    }

    /**
     * The sorter has to know how to sort the columns. It takes the natural
     * order defined on the class. If its Object, it just compares it as
     * strings.
     * @param col the column.
     * @return the class of the column.
     */
    @Override
    public final Class<?> getColumnClass(final int col) {
        switch (col) {
        case COL_TITLE:
            return String.class;
        case COL_RATING:
            return Float.class;
        case COL_YEAR:
            return Integer.class;
        case COL_DURATION:
            return Integer.class;
        case COL_DIRECTOR:
            return String.class;
        case COL_GENRES:
            return Object.class;
        default:
            return Object.class;
        }
    }

    /** Removes the given row from the table model.
     * @param rowToDelete the row which should be deleted. */
    public final void removeRow(final int rowToDelete) {
        Object[][] newMovies =
            new Object[this.getRowCount() - 1][this.getColumnCount() + 1];

        for (int i = 0; i < this.getRowCount(); i++) {
            if (i != rowToDelete) {
                for (int j = 0; j < this.getColumnCount() + 1; j++) {
                    if (i > rowToDelete) {
                        newMovies[i - 1][j] = movies[i][j];
                    } else {
                        newMovies[i][j] = movies[i][j];
                    }
                }
            }
        }

        movies = newMovies;
    }

    /** We need to be able to set the content of the table from outside.
     * @param newMovies the new movies. */
    public final void setTableContent(final HashMap<String, Movie> newMovies) {
        int numberOfRows = newMovies.keySet().size();
        int numberOfCols = this.getColumnCount();
        // We take numberOfCols+1 because we also want to have the complete
        // movie object.
        this.movies = new Object[numberOfRows][numberOfCols + 1];

        int row = 0;
        for (String key : newMovies.keySet()) {
            Movie movie = newMovies.get(key);
            movies[row][COL_TITLE] = movie.getTitle();
            movies[row][COL_RATING] = movie.getRating();
            movies[row][COL_YEAR] = movie.getYear();
            movies[row][COL_DURATION] = movie.getDuration();
            movies[row][COL_DIRECTOR] = movie.getDirector();
            movies[row][COL_GENRES] = movie.getGenres();

            // The complete movie object.
            movies[row][COL_MOVIE] = movie;

            row++;
        }

    }
}
