package movieorganizer.userinterface;
import java.io.StringReader;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/** This class contains helper-methods for generating XML-document-objects. */
public final class XMLHelper {
    /** This class should not be instantiated. */
    private XMLHelper() {
    }

    /**
     * Takes a list of strings which have a certain pattern created by
     * the stromboli-server.
     * @param titles the html-code containing the titles.
     * @return the valid XML-String.
     */
    public static String generateXMLFromTitles(final ArrayList<String> titles) {
        String result = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
        result += "<result>\n";
        for (int i = 0; i < titles.size(); i++) {
            result += "<title>" + titles.get(i) + "</title>\n";
        }
        result += "</result>";

        return result;
    }

    /** Creates an XML Document from a given XML String.
     * @param validXMLString the valid xml-string with the imdb-entries.
     * @return the XML-document generated from the xml-String. */
    public static Document parseMovieXmlFile(final String validXMLString) {
        Document doc = null;
        DocumentBuilderFactory docFactory = DocumentBuilderFactory
                .newInstance();
        try {
            // We need a DocumentBuilder, we get it from the docFactory.
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

            // Parse the XML-code to a DOM object.
            // doc = docBuilder.parse("testMovie.xml");
            doc = docBuilder.parse(new InputSource(new StringReader(
                    validXMLString)));
        } catch (Exception e) {
            System.out.println(e);
        }
        return doc;
    }

    /** Extract all IMDB-entries from a given XML Document.
     * @param validXML a valid XML-Document.
     * @return  the extracted IMDB-entries.*/
    public static ArrayList<ImdbEntry> extractImdbEntries(
            final Document validXML) {
        ArrayList<ImdbEntry> result = new ArrayList<ImdbEntry>();

        // We need the root element.
        Element root = validXML.getDocumentElement();

        // We might receive more than one movie.
        NodeList movies = root.getElementsByTagName("title");
        for (int i = 0; movies != null && movies.getLength() > 0
                && i < movies.getLength(); i++) {
            // Get the movie element.
            Element movie = (Element) movies.item(i);

            // Extract the information.
            String title = getTextValueFromTag(movie, "show", "title");
            int year = getIntValueFromTag(movie, "show", "year");
            if (year == -1) {
                year = Settings.DEF_YEAR;
            }
            // TODO(mirko): extract the other information from the XML.
            String director = getTextValueFromTag(movie, "show", "director");
            Float rating = Settings.DEF_RATING;
            ArrayList<String> genres = new ArrayList<String>();
            int duration = Settings.DEF_DURATION;

            ImdbEntry imdbEntry = new ImdbEntry(title, director, year, rating,
                    genres, duration);
            result.add(imdbEntry);
        }

        return result;
    }

    /** Gets the text from the requested tag with the given field attribute.
     * @param title the title.
     * @param tag the tag.
     * @param attributeName the attribute-name.
     * @return the text. */
    private static String getTextValueFromTag(final Element title,
            final String tag, final String attributeName) {
        NodeList nl = title.getElementsByTagName(tag);
        if (nl != null) {
            for (int i = 0; i < nl.getLength(); i++) {
                Element element = (Element) nl.item(i);
                if (element.getAttribute("field").equals(attributeName)) {
                    return element.getTextContent();
                }
            }
        }
        return "error";
    }

    /** Extracts an Integer using getTextValueFromTag.
     * @param title the title.
     * @param tag the tag.
     * @param attributeName the attribute-name.
     * @return if possible the integer-value, otherwise -1. */
    private static int getIntValueFromTag(final Element title, final String tag,
            final String attributeName) {
        int result = -1;
        try {
            result = Integer.parseInt(getTextValueFromTag(title, tag,
                    attributeName));
        } catch (NumberFormatException e) {
            //System.out.println(e);
            result = -1;
        }
        return result;
    }

    /** Extracts a Float using getTextValueFromTag.
     * @param title the title.
     * @param tag the tag.
     * @param attributeName the attribute-name.
     * @return if possible the float-value, otherwise -1. */
    private static Float getFloatValueFromTag(final Element title,
            final String tag, final String attributeName) {
        float result = -1;
        try {
            result = Float.parseFloat(getTextValueFromTag(title, tag,
                    attributeName));
        } catch (NumberFormatException e) {
            //System.out.println(e);
            result = -1;
        }
        return result;
    }

    // TODO(mirko): implement this correctly.
    /** Extract all genres from a movie given as a DOM object.
     * @param movie the movie.
     * @return if possible the genres, otherwise an empty list. */
    private static ArrayList<String> getGenresFromMovie(final Element movie) {
        ArrayList<String> result = new ArrayList<String>();
        NodeList genres = movie.getElementsByTagName("genre");
        for (int i = 0; genres != null && genres.getLength() > 0
                && i < genres.getLength(); i++) {
            Element element = (Element) genres.item(i);
            String genre = element.getFirstChild().getNodeValue();
            result.add(genre);
        }
        return result;
    }

    /** Extract the text value from a child "tag" from a given XML element.
     * @param element the element.
     * @param tag the tag.
     * @return the text.
     */
    public static String getTextValue(final Element element, final String tag) {
        String result = null;
        NodeList nl = element.getElementsByTagName(tag);
        if (nl != null && nl.getLength() > 0) {
            Element tagElement = (Element) nl.item(0);
            result = tagElement.getFirstChild().getNodeValue();
//            System.out.println("Result: "
//                    + tagElement.getFirstChild().getNodeValue());
        }
        return result;
    }

    /** Converts a String to a float if possible. Otherwise returns -1.
     * @param str the string.
     * @return if possible the float-value, otherwise -1. */
    public static float toFloat(final String str) {
        float result = -1;
        try {
            result = Float.parseFloat(str);
        } catch (Exception e) {
            result = -1;
        }
        return result;
    }

    /** Converts a String to an Integer if possible. Otherwise returns -1.
     * @param str the string.
     * @return if possible the int-value, otherwise -1. */
    public static int toInt(final String str) {
        int result = -1;
        try {
            result = Integer.parseInt(str);
        } catch (Exception e) {
            result = -1;
        }
        return result;
    }
}
